@@ -10,7 +10,7 @@ import time
 
 class Test_sims:
 
-    device = torch.device('cuda:1' if cuda.is_available() else 'cpu')
+    device = torch.device('cuda' if cuda.is_available() else 'cpu')
     # device = torch.device('cpu')
     # dtype, atol = torch.float32, 1e-4
     dtype, atol = torch.float64, 1e-9
@@ -61,12 +61,12 @@ class Test_sims:
                         10*torch.atan(t - round(nT/2))/π], 1)  # (1,xyz,nT)
 
         # rf.requires_grad, gr.requires_grad = True, True
-        beff = beffective.rfgr2beff(rf, gr, loc, Δf, b1Map, γ)
+        beff = beffective.rfgr2beff(rf, gr, loc, Δf=Δf, b1Map=b1Map, γ=γ)
         beff.requires_grad = True
 
         # Check handling of 1-coil `rf`, `b1Map` that omitted the `nCoils` dim
-        beff_missing_dim = beffective.rfgr2beff(rf[..., 0], gr, loc, Δf,
-                                                b1Map[..., 0], γ)
+        beff_missing_dim = beffective.rfgr2beff(rf[..., 0], gr, loc, Δf=Δf,
+                                                b1Map=b1Map[..., 0], γ=γ)
 
         # %% sim
         print('\nblochsim tests:')
